.row{display:flex;flex-direction:row;align-items:center;}.column{display:flex;flex-direction:column;}.pl5{padding-left:5px;}.pl10{padding-left:10px;}.pl15{padding-left:15px;}.pl20{padding-left:20px;}.pr5{padding-right:5px;}.pr10{padding-right:10px;}.pr15{padding-right:15px;}.pr20{padding-right:20px;}.pt5{padding-top:5px;}.pt10{padding-top:10px;}.pt15{padding-top:15px;}.pt20{padding-top:20px;}.pb5{padding-bottom:5px;}.pb10{padding-bottom:10px;}.pb15{padding-bottom:15px;}.pb20{padding-bottom:20px;}.jp-Cell-outputWrapper{margin-top:0px!important;}.jp-MarkdownOutput{overflow:visible!important;}.jp-Cell{padding:0px!important;}.jp-RenderedHTMLCommon h1:first-child,.jp-RenderedHTMLCommon h2:first-child,.jp-RenderedHTMLCommon h3:first-child,.jp-RenderedHTMLCommon h4:first-child,.jp-RenderedHTMLCommon h5:first-child,.jp-RenderedHTMLCommon h6:first-child{margin-top:0px!important;}.jp-RenderedHTMLCommon h1:last-child,.jp-RenderedHTMLCommon h2:last-child,.jp-RenderedHTMLCommon h3:last-child,.jp-RenderedHTMLCommon h4:last-child,.jp-RenderedHTMLCommon h5:last-child,.jp-RenderedHTMLCommon h6:last-child{margin-bottom:0px!important;}.jp-OutputArea-output{overflow:hidden!important;}